// nfapinet.h

#pragma once

using namespace System;

namespace nfapinet 
{
	public enum class NF_STATUS
	{
		NF_STATUS_SUCCESS		= 0,
		NF_STATUS_FAIL			= -1,
		NF_STATUS_INVALID_ENDPOINT_ID	= -2,
		NF_STATUS_NOT_INITIALIZED	= -3,
		NF_STATUS_IO_ERROR		= -4
	};

	public enum class NF_DIRECTION
	{
		NF_D_IN = 1,		// Incoming TCP connection or UDP packet
		NF_D_OUT = 2,		// Outgoing TCP connection or UDP packet
		NF_D_BOTH = 3		// Any direction
	};

	public enum class NF_FILTERING_FLAG
	{
		NF_ALLOW = 0,		// Allow the activity without filtering transmitted packets
		NF_BLOCK = 1,		// Block the activity
		NF_FILTER = 2,		// Filter the transmitted packets
		NF_SUSPENDED = 4,	// Suspend receives from server and sends from client
		NF_OFFLINE = 8,		// Emulate establishing a TCP connection with remote server
		NF_INDICATE_CONNECT_REQUESTS = 16 // Indicate outgoing connect requests to API
	};

    public enum class NF_CONSTS
    {
		NF_MAX_ADDRESS_LENGTH = 28,
		NF_MAX_IP_ADDRESS_LENGTH = 16
    };

	/**
	*	Filtering rule
	**/
	public ref struct NF_RULE
	{
		int				protocol;	// IPPROTO_TCP or IPPROTO_UDP        
		unsigned long	processId;	// Process identifier
		unsigned char	direction;	// See NF_DIRECTION
		unsigned short	localPort;	// Local port
		unsigned short	remotePort;	// Remote port
		unsigned short	ip_family;	// AF_INET for IPv4 and AF_INET6 for IPv6
		
		// Local IP (or network if localIpAddressMask is not zero)
		cli::array<unsigned char>	^ localIpAddress;	
		
		// Local IP mask
		cli::array<unsigned char>	^ localIpAddressMask; 
		
		// Remote IP (or network if remoteIpAddressMask is not zero)
		cli::array<unsigned char>	^ remoteIpAddress; 
		
		// Remote IP mask
		cli::array<unsigned char>	^ remoteIpAddressMask; 

		unsigned long	filteringFlag;	// See NF_FILTERING_FLAG
	};


	/**
	*	TCP connection properties
	**/
	public ref struct NF_TCP_CONN_INFO
	{
		unsigned long	filteringFlag;	// See NF_FILTERING_FLAG
		unsigned long	processId;		// Process identifier
		unsigned char	direction;		// See NF_DIRECTION
		unsigned short	ip_family;		// AF_INET for IPv4 and AF_INET6 for IPv6
		
		// Local address as sockaddr_in for IPv4 and sockaddr_in6 for IPv6
		cli::array<unsigned char> ^ localAddress; 
		
		// Remote address as sockaddr_in for IPv4 and sockaddr_in6 for IPv6
		cli::array<unsigned char> ^ remoteAddress;

	};

	/**
	*	UDP endpoint properties
	**/
	public ref struct NF_UDP_CONN_INFO
	{
		unsigned long	processId;		// Process identifier
		unsigned short	ip_family;		// AF_INET for IPv4 and AF_INET6 for IPv6
		
		// Local address as sockaddr_in for IPv4 and sockaddr_in6 for IPv6
		cli::array<unsigned char> ^ localAddress; 
	};

	/**
	*	UDP options
	**/
	public ref struct NF_UDP_OPTIONS
	{
		unsigned long	flags;		// UDP flags
		cli::array<unsigned char> ^ options; // Options array
	};

	/**
	*	UDP connect request properties
	**/
	public ref struct NF_UDP_CONN_REQUEST
	{
		unsigned long	filteringFlag;	// See NF_FILTERING_FLAG
		unsigned long	processId;		// Process identifier
		unsigned short	ip_family;		// AF_INET for IPv4 and AF_INET6 for IPv6
		
		// Local address as sockaddr_in for IPv4 and sockaddr_in6 for IPv6
		cli::array<unsigned char> ^ localAddress; 

		// Remote address as sockaddr_in for IPv4 and sockaddr_in6 for IPv6
		cli::array<unsigned char> ^ remoteAddress;
	};

	typedef unsigned __int64 ENDPOINT_ID;

	// Managed event handler interface
	public interface class NF_EventHandler
	{
	public:
		void threadStart();
		void threadEnd();
		void tcpConnectRequest(ENDPOINT_ID id, NF_TCP_CONN_INFO ^ pConnInfo);
		void tcpConnected(ENDPOINT_ID id, NF_TCP_CONN_INFO ^ pConnInfo);
		void tcpClosed(ENDPOINT_ID id, NF_TCP_CONN_INFO ^ pConnInfo);
		void tcpReceive(ENDPOINT_ID id, const unsigned char * buf, int len);
		void tcpSend(ENDPOINT_ID id, const unsigned char * buf, int len);
		void tcpCanReceive(ENDPOINT_ID id);
		void tcpCanSend(ENDPOINT_ID id);
		void udpCreated(ENDPOINT_ID id, NF_UDP_CONN_INFO ^ pConnInfo);
		void udpConnectRequest(ENDPOINT_ID id, NF_UDP_CONN_REQUEST ^ pConnReq);
		void udpClosed(ENDPOINT_ID id, NF_UDP_CONN_INFO ^ pConnInfo);
		void udpReceive(ENDPOINT_ID id, cli::array<unsigned char> ^ remoteAddress, const unsigned char * buf, int len, nfapinet::NF_UDP_OPTIONS ^ options);
		void udpSend(ENDPOINT_ID id, cli::array<unsigned char> ^ remoteAddress, const unsigned char * buf, int len, nfapinet::NF_UDP_OPTIONS ^ options);
		void udpCanReceive(ENDPOINT_ID id);
		void udpCanSend(ENDPOINT_ID id);
	};

	// Managed wrapper over API 
	public ref class NFAPI
	{
	public:
		/**
		* Initializes the internal data structures and starts the filtering thread.
		* @param driverName The name of TDI hooking driver, without ".sys" extension.
		* @param pHandler Pointer to event handling object
		**/
		static NF_STATUS nf_init(System::String^ driverName, nfapinet::NF_EventHandler ^ pHandler);

		/**
		* Stops the filtering thread, breaks all filtered connections and closes
		* a connection with the hooking driver.
		**/
		static void nf_free();

		/**
		* Registers and starts a driver with specified name (without ".sys" extension)
		* @param driverName 
		**/
		static NF_STATUS nf_registerDriver(System::String^ driverName);

		/**
		* Unregisters a driver with specified name (without ".sys" extension)
		* @param driverName 
		**/
		static NF_STATUS nf_unRegisterDriver(System::String^ driverName);


		//
		// TCP control routines
		//

		/**
		* Suspends or resumes indicating of sends and receives for specified connection.
		* @param id Connection identifier
		* @param suspended true for suspend, false for resume 
		**/
		static NF_STATUS nf_tcpSetConnectionState(ENDPOINT_ID id, BOOL suspended);

		/**
		* Sends the buffer to remote server via specified connection.
		* @param id Connection identifier
		* @param buf Pointer to data buffer
		* @param len Buffer length
		**/
		static NF_STATUS nf_tcpPostSend(ENDPOINT_ID id, const unsigned char * buf, int len);

		/**
		* Indicates the buffer to local process via specified connection.
		* @param id Unique connection identifier
		* @param buf Pointer to data buffer
		* @param len Buffer length
		**/
		static NF_STATUS nf_tcpPostReceive(ENDPOINT_ID id, const unsigned char * buf, int len);

		/**
		* Breaks the connection with given id.
		* @param id Connection identifier
		**/
		static NF_STATUS nf_tcpClose(ENDPOINT_ID id);

		//
		// UDP control routines
		//

		/**
		* Suspends or resumes indicating of sends and receives for specified socket.
		* @param id Socket identifier
		* @param suspended true for suspend, false for resume 
		**/
		static NF_STATUS nf_udpSetConnectionState(ENDPOINT_ID id, BOOL suspended);

		/**
		* Sends the buffer to remote server via specified socket.
		* @param id Socket identifier
		* @param remoteAddress Destination address
		* @param buf Pointer to data buffer
		* @param len Buffer length
		**/
		static NF_STATUS nf_udpPostSend(ENDPOINT_ID id, 
			cli::array<unsigned char> ^ remoteAddress, 
			const unsigned char * buf, int len,
			nfapinet::NF_UDP_OPTIONS ^ options);

		/**
		* Indicates the buffer to local process via specified socket.
		* @param id Unique connection identifier
		* @param remoteAddress Source address
		* @param buf Pointer to data buffer
		* @param len Buffer length
		**/
		static NF_STATUS nf_udpPostReceive(ENDPOINT_ID id, 
			cli::array<unsigned char> ^ remoteAddress, 
			const unsigned char * buf, int len,
			nfapinet::NF_UDP_OPTIONS ^ options);

		//
		// Filtering rules 
		//

		/**
		* Add a rule to the head of rules list in driver.
		* @param pRule See <tt>NF_RULE</tt>
		* @param toHead TRUE (1) - add rule to list head, FALSE (0) - add rule to tail
		**/
		static NF_STATUS nf_addRule(NF_RULE ^ pRule, int toHead);
		
		/**
		* Removes all rules from driver.
		**/
		static NF_STATUS nf_deleteRules();

		/**
		 *	Sets the timeout for TCP connections and returns old timeout.
		 *	@param timeout Timeout value in milliseconds. Specify zero value to disable timeouts.
		 */
		static unsigned long nf_setTCPTimeout(unsigned long timeout);

		/**
		 *	Disables indicating TCP packets to user mode for the specified endpoint
		 *  @param id Socket identifier
		 */
		static NF_STATUS nf_tcpDisableFiltering(ENDPOINT_ID id);

		/**
		 *	Disables indicating UDP packets to user mode for the specified endpoint
		 *  @param id Socket identifier
		 */
		static NF_STATUS nf_udpDisableFiltering(ENDPOINT_ID id);

		/**
		 *	Disables filtering for all filtered TCP/UDP sockets
		 */
		static NF_STATUS nf_disableFiltering();

		static System::String^ nf_getProcessName(unsigned long processId);
	};
}
